@@ -1,5 +1,6 @@
 import contextlib
 import copy
+import warnings
 
 import numpy as np
 
@@ -8,12 +9,12 @@ from skrobot.coordinates.math import _check_valid_rotation
 from skrobot.coordinates.math import _check_valid_translation
 from skrobot.coordinates.math import _wrap_axis
 from skrobot.coordinates.math import angle_between_vectors
+from skrobot.coordinates.math import cross_product
 from skrobot.coordinates.math import matrix2quaternion
 from skrobot.coordinates.math import matrix_log
 from skrobot.coordinates.math import normalize_vector
 from skrobot.coordinates.math import quaternion2matrix
 from skrobot.coordinates.math import quaternion_multiply
-from skrobot.coordinates.math import quaternion_normalize
 from skrobot.coordinates.math import random_rotation
 from skrobot.coordinates.math import random_translation
 from skrobot.coordinates.math import rotate_matrix
@@ -66,15 +67,105 @@ def transform_coords(c1, c2, out=None):
            [-5.00000000e-01,  8.66025404e-01,  2.77555756e-17]])
     """
     if out is None:
-        out = Coordinates()
+        out = Coordinates(check_validity=False)
     elif not isinstance(out, Coordinates):
         raise TypeError("Input type should be skrobot.coordinates.Coordinates")
-    out.translation = c1.translation + np.dot(c1.rotation, c2.translation)
-    out.rotation = quaternion_normalize(
-        quaternion_multiply(c1.quaternion, c2.quaternion))
+    out._translation = c1.translation + np.dot(c1.rotation, c2.translation)
+    out._rotation = np.matmul(c1.rotation, c2.rotation)
     return out
 
 
+class Transform(object):
+    """Transform specified by translation and rotation
+
+    Parameters
+    ----------
+    translation : list(3,) or numpy.ndarray(3,)
+        translation
+    rot : numpy.ndarray(3, 3)
+        3x3 rotation matrix
+    """
+
+    def __init__(self, translation, rotation):
+        self.translation = np.array(translation)
+        self.rotation = rotation
+
+    def transform_vector(self, vec):
+        """Apply this transform to vector/vectors
+
+        Parameters
+        ----------
+        vec : numpy.ndarray(3,) or numpy.ndarray(n_points, 3)
+            vector/vectors to be transformed
+
+        Returns
+        -------
+        vec_transformed : numpy.ndarray(3,) or numpy.ndarray(n_points, 3)
+            transformed points
+        """
+        assert vec.ndim < 3, "vec must be either 1 or 2 dimensional."
+        if vec.ndim == 1:
+            return self.rotation.dot(vec.T) + self.translation
+        if vec.ndim == 2:
+            return self.rotation.dot(vec.T).T + self.translation[None, :]
+
+    def rotate_vector(self, vec):
+        """Rotate 3-dimensional vector using rotation of this Transform
+
+        Parameters
+        ----------
+        vec : numpy.ndarray(3,) or numpy.ndarray(3, n_points)
+            vector (or vectors) to be roated
+
+        Returns
+        -------
+        vec_transformed : numpy.ndarray(3,), numpy.ndarray(3, n_points)
+            rotated vector (or vectors)
+        """
+        assert vec.ndim < 3, "vec must be either 1 or 2 dimensional."
+        if vec.ndim == 1:
+            return self.rotation.dot(vec.T)
+        if vec.ndim == 2:
+            return self.rotation.dot(vec.T).T
+
+    def inverse_transformation(self):
+        """Return inverse transform
+
+        Returns
+        -------
+        inv_transform : skrobot.coordinates.base.Transform
+            inverse transformation
+        """
+        new_rot = self.rotation.T
+        new_trans = -new_rot.dot(self.translation)
+        return Transform(new_trans, new_rot)
+
+    def __mul__(self, tf_23):
+        """Composite this transform with other transform
+
+        Parameters
+        ----------
+        tf_23 : skrobot.coordinates.base.Transform
+            the other transform.
+
+        Returns
+        -------
+        tf_13 : skrobot.coordinates.base.Transform
+            Let this (self) transform as tf_12, then with the
+            other transform tf_23, we obtain tf_13 = tf_12 * tf_23
+        """
+        tf_12 = self
+        tran_12, rot_12 = tf_12.translation, tf_12.rotation
+        tran_23, rot_23 = tf_23.translation, tf_23.rotation
+        rot_13 = rot_23.dot(rot_12)
+        tran_13 = tran_23 + rot_23.dot(tran_12)
+        tf_13 = Transform(tran_13, rot_13)
+        return tf_13
+
+    def __rmul__(self, other):
+        return other.__mul__(self)
+
+
 class Coordinates(object):
 
     """Coordinates class to manipulate rotation and translation.
@@ -90,22 +181,31 @@ class Coordinates(object):
         we can take 3x3 rotation matrix or
         [yaw, pitch, roll] or
         quaternion [w, x, y, z] order
-    name : string or None
+    name : str or None
         name of this coordinates
+    check_validity : bool (optional)
+        Default `True`.
+        If this value is `True`, check whether an input rotation
+        and an input translation are valid.
     """
 
     def __init__(self,
                  pos=[0, 0, 0],
                  rot=np.eye(3),
                  name=None,
-                 hook=None):
-        if (isinstance(pos, list) or isinstance(pos, np.ndarray)):
-            T = np.array(pos, dtype=np.float64)
-            if T.shape == (4, 4):
-                pos = T[:3, 3]
-                rot = T[:3, :3]
-        self.rotation = rot
-        self.translation = pos
+                 hook=None,
+                 check_validity=True):
+        if check_validity:
+            if (isinstance(pos, list) or isinstance(pos, np.ndarray)):
+                T = np.array(pos, dtype=np.float64)
+                if T.shape == (4, 4):
+                    pos = T[:3, 3]
+                    rot = T[:3, :3]
+            self.rotation = rot
+            self.translation = pos
+        else:
+            self._rotation = rot
+            self._translation = pos
         if name is None:
             name = ''
         self.name = name
@@ -121,13 +221,23 @@ class Coordinates(object):
         finally:
             self._hook = hook
 
+    def get_transform(self):
+        """Return Transform object
+
+        Returns
+        -------
+        transform : skrobot.coordinates.base.Transform
+            corrensponding Transform to this coordinates
+        """
+        return Transform(self.worldpos(), self.worldrot())
+
     @property
     def rotation(self):
         """Return rotation matrix of this coordinates.
 
         Returns
         -------
-        self._rotation : np.ndarray
+        self._rotation : numpy.ndarray
             3x3 rotation matrix
 
         Examples
@@ -156,7 +266,7 @@ class Coordinates(object):
 
         Parameters
         ----------
-        rotation : list or np.ndarray
+        rotation : list or numpy.ndarray
             we can take 3x3 rotation matrix or
             rpy angle [yaw, pitch, roll] or
             quaternion [w, x, y, z] order
@@ -164,18 +274,16 @@ class Coordinates(object):
         rotation = np.array(rotation)
         # Convert quaternions
         if rotation.shape == (4,):
-            self._q = np.array([q for q in rotation])
-            if np.abs(np.linalg.norm(self._q) - 1.0) > 1e-3:
+            q = np.array([q for q in rotation])
+            if np.abs(np.linalg.norm(q) - 1.0) > 1e-3:
                 raise ValueError('Invalid quaternion. Must be '
                                  'norm 1.0, get {}'.
-                                 format(np.linalg.norm(self._q)))
-            rotation = quaternion2matrix(self._q)
+                                 format(np.linalg.norm(q)))
+            rotation = quaternion2matrix(q)
         elif rotation.shape == (3,):
             # Convert [yaw-pitch-roll] to rotation matrix
-            self._q = rpy2quaternion(rotation)
-            rotation = quaternion2matrix(self._q)
-        else:
-            self._q = matrix2quaternion(rotation)
+            q = rpy2quaternion(rotation)
+            rotation = quaternion2matrix(q)
 
         # Convert lists and tuples
         if type(rotation) in (list, tuple):
@@ -190,7 +298,7 @@ class Coordinates(object):
 
         Returns
         -------
-        self._translation : np.ndarray
+        self._translation : numpy.ndarray
             vector shape of (3, ). unit is [m]
 
         Examples
@@ -214,7 +322,7 @@ class Coordinates(object):
 
         Parameters
         ----------
-        translation : list or tuple or np.ndarray
+        translation : list or tuple or numpy.ndarray
             shape of (3,) translation vector
         """
         # Convert lists to translation arrays
@@ -230,7 +338,7 @@ class Coordinates(object):
 
         Returns
         -------
-        self._name : string
+        self._name : str
             name of this coordinate
         """
         return self._name
@@ -241,7 +349,7 @@ class Coordinates(object):
 
         Parameters
         ----------
-        name : string
+        name : str
             name of this coordinate
         """
         if not isinstance(name, str):
@@ -280,9 +388,9 @@ class Coordinates(object):
 
         Parameters
         ----------
-        vec : list or np.ndarray
+        vec : list or numpy.ndarray
             shape of (3,) translation vector. unit is [m] order.
-        wrt : string or Coordinates (optional)
+        wrt : str or Coordinates (optional)
             translate with respect to wrt.
 
         Examples
@@ -313,7 +421,8 @@ class Coordinates(object):
         vec = np.array(vec, dtype=np.float64)
         return self.newcoords(
             self.rotation,
-            self.parent_orientation(vec, wrt) + self.translation)
+            self.parent_orientation(vec, wrt) + self.translation,
+            check_validity=False)
 
     def transform_vector(self, v):
         """"Return vector represented at world frame.
@@ -384,7 +493,7 @@ class Coordinates(object):
             result of inverse transformation.
         """
         if dest is None:
-            dest = Coordinates()
+            dest = Coordinates(check_validity=False)
         dest.rotation = self.rotation.T
         dest.translation = np.matmul(dest.rotation, self.translation)
         dest.translation = -1.0 * dest.translation
@@ -414,7 +523,7 @@ class Coordinates(object):
 
         Returns
         -------
-        matrix : np.ndarray
+        matrix : numpy.ndarray
             homogeneous transformation matrix shape of (4, 4)
 
         Examples
@@ -450,7 +559,7 @@ class Coordinates(object):
 
         Returns
         -------
-        self._q : np.ndarray
+        q : numpy.ndarray
             [w, x, y, z] quaternion
 
         Examples
@@ -464,7 +573,7 @@ class Coordinates(object):
         >>> c.quaternion
         array([0.8236391 , 0.1545085 , 0.47552826, 0.26761657])
         """
-        return self._q
+        return matrix2quaternion(self._rotation)
 
     @property
     def dual_quaternion(self):
@@ -498,11 +607,12 @@ class Coordinates(object):
 
         Parameters
         ----------
-        v : np.ndarray
+        v : numpy.ndarray
             vector shape of (3,)
 
-        Returns:
-        np.matmul(self.rotation, v) : np.ndarray
+        Returns
+        -------
+        np.matmul(self.rotation, v) : numpy.ndarray
             rotated vector
 
         Examples
@@ -513,24 +623,24 @@ class Coordinates(object):
         >>> c.rotate_vector([1, 2, 3])
         array([-1., -2.,  3.])
         """
-        return np.matmul(self.rotation, v)
+        return np.matmul(self._rotation, v)
 
     def inverse_rotate_vector(self, v):
-        return np.matmul(v, self.rotation)
+        return np.matmul(v, self._rotation)
 
     def transform(self, c, wrt='local'):
-        """Transform this coordinate by coords based on wrt
+        """Transform this coordinates by coords based on wrt
 
-        Note that this function changes this coordinate's
+        Note that this function changes this coordinates
         translation and rotation.
-        If you would like not to change this coordinate,
+        If you would like not to change this coordinates,
         Please use copy_worldcoords()
 
         Parameters
         ----------
         c : skrobot.coordinates.Coordinates
             coordinate
-        wrt : string or skrobot.coordinates.Coordinates
+        wrt : str or skrobot.coordinates.Coordinates
             If wrt is 'local' or self, multiply c from the right.
             If wrt is 'world' or 'parent' or self.parent,
             transform c with respect to worldcoord.
@@ -583,7 +693,7 @@ class Coordinates(object):
 
         Returns
         -------
-        rpy_angle(self.rotation) : tuple of np.ndarray
+        rpy_angle(self.rotation) : tuple(numpy.ndarray, numpy.ndarray)
             a pair of rpy angles. See also skrobot.coordinates.math.rpy_angle
 
         Examples
@@ -694,9 +804,9 @@ class Coordinates(object):
             a1 = coords1.axis(ax)
             a1_mirror = - a1
             dr1 = angle_between_vectors(a0, a1, normalize=False) \
-                * normalize_vector(np.cross(a0, a1))
+                * normalize_vector(cross_product(a0, a1))
             dr1m = angle_between_vectors(a0, a1_mirror, normalize=False) \
-                * normalize_vector(np.cross(a0, a1_mirror))
+                * normalize_vector(cross_product(a0, a1_mirror))
             return np.linalg.norm(dr1) < np.linalg.norm(dr1m)
 
         if rotation_axis in ['x', 'y', 'z']:
@@ -708,7 +818,7 @@ class Coordinates(object):
                 dif_rot = np.matmul(
                     self.worldrot().T,
                     angle_between_vectors(a0, a1, normalize=False)
-                    * normalize_vector(np.cross(a0, a1)))
+                    * normalize_vector(cross_product(a0, a1)))
         elif rotation_axis in ['xx', 'yy', 'zz']:
             ax = rotation_axis[0]
             a0 = self.axis(ax)
@@ -718,7 +828,7 @@ class Coordinates(object):
             dif_rot = np.matmul(
                 self.worldrot().T,
                 angle_between_vectors(a0, a2, normalize=False)
-                * normalize_vector(np.cross(a0, a2)))
+                * normalize_vector(cross_product(a0, a2)))
         elif rotation_axis in ['xy', 'yx', 'yz', 'zy', 'zx', 'xz']:
             if rotation_axis in ['xy', 'yx']:
                 ax1 = 'z'
@@ -734,7 +844,7 @@ class Coordinates(object):
             dif_rot = np.matmul(
                 self.worldrot().T,
                 angle_between_vectors(a0, a1, normalize=False)
-                * normalize_vector(np.cross(a0, a1)))
+                * normalize_vector(cross_product(a0, a1)))
             norm = np.linalg.norm(dif_rot)
             if np.isclose(norm, 0.0):
                 self_coords = self.copy_worldcoords()
@@ -745,7 +855,7 @@ class Coordinates(object):
             dif_rot = np.matmul(
                 self_coords.worldrot().T,
                 angle_between_vectors(a0, a1, normalize=False)
-                * normalize_vector(np.cross(a0, a1)))
+                * normalize_vector(cross_product(a0, a1)))
         elif rotation_axis in ['xm', 'ym', 'zm']:
             rot = coords.worldrot()
             ax = rotation_axis[0]
@@ -769,9 +879,10 @@ class Coordinates(object):
 
         Parameters
         ----------
-        mat : np.ndarray
+        mat : numpy.ndarray
             rotation matrix shape of (3, 3)
-        wrt : string or skrobot.coordinates.Coordinates
+        wrt : str or skrobot.coordinates.Coordinates
+            with respect to.
 
         Returns
         -------
@@ -779,18 +890,18 @@ class Coordinates(object):
         """
         if wrt == 'local' or wrt == self:
             rot = np.matmul(self.rotation, mat)
-            self.newcoords(rot, self.translation)
+            self.newcoords(rot, self.translation, check_validity=False)
         elif wrt == 'parent' or wrt == self.parent or \
                 wrt == 'world' or wrt is None or \
                 wrt == worldcoords:
             rot = np.matmul(mat, self.rotation)
-            self.newcoords(rot, self.translation)
+            self.newcoords(rot, self.translation, check_validity=False)
         elif isinstance(wrt, Coordinates):
             r2 = wrt.worldrot()
             r2t = r2.T
             r2t = np.matmul(mat, r2t)
             r2t = np.matmul(r2, r2t)
-            self.rotation = np.matmul(r2t, self.rotation)
+            self._rotation = np.matmul(r2t, self.rotation)
         else:
             raise ValueError('wrt {} is not supported'.format(wrt))
         return self
@@ -798,15 +909,39 @@ class Coordinates(object):
     def rotate(self, theta, axis=None, wrt='local'):
         """Rotate this coordinate by given theta and axis.
 
+        This coordinate system is rotated relative to theta radians
+        around the `axis` axis.
+        Note that this function does not change a position of this coordinate.
+        If you want to rotate this coordinates around with world frame,
+        you can use `transform` function.
+        Please see examples.
+
         Parameters
         ----------
         theta : float
-            radian
-        wrt : string or skrobot.coordinates.Coordinates
+            relartive rotation angle in radian.
+        axis : str or None or numpy.ndarray
+            axis of rotation.
+            The value of `axis` is represented as `wrt` frame.
+        wrt : str or skrobot.coordinates.Coordinates
 
         Returns
         -------
         self : skrobot.coordinates.Coordinates
+
+        Examples
+        --------
+        >>> from skrobot.coordinates import Coordinates
+        >>> from numpy import pi
+        >>> c = Coordinates()
+        >>> c.translate((1.0, 0, 0))
+        >>> c.rotate(pi / 2.0, 'z', wrt='local')
+        >>> c.translation
+        array([1., 0., 0.])
+
+        >>> c.transform(Coordinates().rotate(np.pi / 2.0, 'z'), wrt='world')
+        >>> c.translation
+        array([0., 1., 0.])
         """
         if isinstance(axis, list) or isinstance(axis, np.ndarray):
             self.rotate_with_matrix(
@@ -814,16 +949,17 @@ class Coordinates(object):
         elif axis is None or axis is False:
             self.rotate_with_matrix(theta, wrt)
         elif wrt == 'local' or wrt == self:
-            self.rotation = rotate_matrix(self.rotation, theta, axis)
+            self._rotation = rotate_matrix(self.rotation, theta, axis)
         elif wrt == 'parent' or wrt == 'world':
-            self.rotation = rotate_matrix(self.rotation, theta,
-                                          axis, True)
+            self._rotation = rotate_matrix(self.rotation, theta,
+                                           axis, True)
         elif isinstance(wrt, Coordinates):  # C1'=C2*R*C2(-1)*C1
             self.rotate_with_matrix(
                 rotation_matrix(theta, axis), wrt)
         else:
             raise ValueError('wrt {} not supported'.format(wrt))
-        return self.newcoords(self.rotation, self.translation)
+        return self.newcoords(self.rotation, self.translation,
+                              check_validity=False)
 
     def copy(self):
         """Return a deep copy of the Coordinates."""
@@ -831,8 +967,9 @@ class Coordinates(object):
 
     def copy_coords(self):
         """Return a deep copy of the Coordinates."""
-        return Coordinates(pos=copy.deepcopy(self.worldpos()),
-                           rot=copy.deepcopy(self.worldrot()))
+        return Coordinates(pos=np.copy(self.worldpos()),
+                           rot=np.copy(self.worldrot()),
+                           check_validity=False)
 
     def coords(self):
         """Return a deep copy of the Coordinates."""
@@ -854,7 +991,7 @@ class Coordinates(object):
 
         Returns
         -------
-        self.rotation : np.ndarray
+        self.rotation : numpy.ndarray
             rotation matrix of this coordinate
         """
         return self.rotation
@@ -866,19 +1003,39 @@ class Coordinates(object):
 
         Returns
         -------
-        self.translation : np.ndarray
+        self.translation : numpy.ndarray
             translation of this coordinate
         """
         return self.translation
 
-    def newcoords(self, c, pos=None):
-        """Update of coords is always done through newcoords."""
+    def newcoords(self, c, pos=None, check_validity=True):
+        """Update of coords is always done through newcoords.
+
+        Parameters
+        ----------
+        c : skrobot.coordinates.Coordinates or numpy.ndarray
+            If pos is `None`, `c` means new Coordinates.
+            If pos is given, `c` means rotation matrix.
+        pos : numpy.ndarray or None
+            new translation.
+        check_validity : bool
+            If this value is `True`, check whether an input rotation
+            and an input translation are valid.
+        """
         if pos is not None:
-            self.rotation = copy.deepcopy(c)
-            self.translation = copy.deepcopy(pos)
+            if check_validity:
+                self.rotation = copy.deepcopy(c)
+                self.translation = copy.deepcopy(pos)
+            else:
+                self._rotation = np.copy(c)
+                self._translation = np.copy(pos)
         else:
-            self.rotation = copy.deepcopy(c.rotation)
-            self.translation = copy.deepcopy(c.translation)
+            if check_validity:
+                self.rotation = copy.deepcopy(c.rotation)
+                self.translation = copy.deepcopy(c.translation)
+            else:
+                self._rotation = np.copy(c.rotation)
+                self._translation = np.copy(c.translation)
         return self
 
     def __mul__(self, other_c):
@@ -956,45 +1113,93 @@ class CascadedCoords(Coordinates):
 
         self._worldcoords = Coordinates(pos=self.translation,
                                         rot=self.rotation,
-                                        hook=self.update)
+                                        hook=self.update,
+                                        check_validity=False)
 
         self.parent = parent
         if parent is not None:
-            self.parent.assoc(self)
+            # Because we must self.parent = parent in this case,
+            # force=True is required.
+            parent.assoc(self, force=True)
 
     @property
     def descendants(self):
         return self._descendants
 
-    def assoc(self, child, c=None):
+    def assoc(self, child, relative_coords=None, force=False,
+              **kwargs):
+        """Associate child coords to this coordinate.
+
+        If `relative_coords` is `None`, the translation and rotation
+        of childcoord in the world coordinate system do not change.
+        If `relative_coords` is specified, childcoord is assoced
+        at translation and rotation of `relative_coords`.
+        By default, if child is already assoced to some other coords,
+        raise an exception. But if `force` is `True`, you can overwrite
+        the existing assoc relation.
+
+        Parameters
+        ----------
+        child : CascadedCoords
+            child coordinate.
+        relative_coords : None or Coordinates
+            child coordinate's relative coordinate.
+        force : bool
+            predicate for overwriting the existing assoc-relation
+
+        Returns
+        -------
+        child : CascadedCoords
+            assoced child.
+        """
+        if 'c' in kwargs:
+            warnings.warn(
+                'Argument `c` is deprecated. '
+                'Please use `relative_coords` instead',
+                DeprecationWarning)
+            relative_coords = kwargs['c']
+
+        is_invalid_assoc = (child.parent is not None) and (not force)
+        if is_invalid_assoc:
+            msg = "child already has an assoc relation with '{0}'."\
+                " To overwrite this, please specify force=True."\
+                .format(child.parent.name)
+            raise RuntimeError(msg)
+
         if not (child in self.descendants):
-            if c is None:
-                c = self.worldcoords().transformation(
+            if relative_coords is None:
+                relative_coords = self.worldcoords().transformation(
                     child.worldcoords())
-            child.obey(self)
-            child.newcoords(c)
+            child.parent = self
+            child.newcoords(relative_coords, check_validity=False)
             self._descendants.append(child)
             return child
 
-    def obey(self, mother):
-        if self.parent is not None:
-            self.parent.dissoc(self)
-        self.parent = mother
-
     def dissoc(self, child):
         if child in self.descendants:
             c = child.worldcoords().copy_coords()
             self._descendants.remove(child)
-            child.disobey(self)
-            child.newcoords(c)
+            child.parent = None
+            child.newcoords(c, check_validity=False)
 
-    def disobey(self, mother):
-        if self.parent == mother:
-            self.parent = None
-        return self.parent
+    def newcoords(self, c, pos=None, check_validity=True):
+        """Update this coordinates.
 
-    def newcoords(self, c, pos=None):
-        super(CascadedCoords, self).newcoords(c, pos)
+        This function records that this CascadedCoords has changed and
+        recursively records the change to descendants of this CascadedCoords.
+
+        Parameters
+        ----------
+        c : skrobot.coordinates.Coordinates or numpy.ndarray
+            If pos is `None`, `c` means new Coordinates.
+            If pos is given, `c` means rotation matrix.
+        pos : numpy.ndarray or None
+            new translation.
+        check_validity : bool
+            If this value is `True`, check whether an input rotation
+            and an input translation are valid.
+        """
+        super(CascadedCoords, self).newcoords(c, pos, check_validity)
         self.changed()
         return self
 
@@ -1017,11 +1222,13 @@ class CascadedCoords(Coordinates):
 
     def rotate_with_matrix(self, matrix, wrt):
         if wrt == 'local' or wrt == self:
-            self.rotation = np.dot(self.rotation, matrix)
-            return self.newcoords(self.rotation, self.translation)
+            self._rotation = np.dot(self.rotation, matrix)
+            return self.newcoords(self.rotation, self.translation,
+                                  check_validity=False)
         elif wrt == 'parent' or wrt == self.parent:
-            self.rotation = np.matmul(matrix, self.rotation)
-            return self.newcoords(self.rotation, self.translation)
+            rotation = np.matmul(matrix, self.rotation)
+            return self.newcoords(
+                rotation, self.translation, check_validity=False)
         else:
             parent_coords = self.parentcoords()
             parent_rot = parent_coords.rotation
@@ -1031,8 +1238,9 @@ class CascadedCoords(Coordinates):
                 matrix = np.matmul(matrix, wrt_rot.T)
             matrix = np.matmul(matrix, parent_rot)
             matrix = np.matmul(parent_rot.T, matrix)
-            self.rotation = np.matmul(matrix, self.rotation)
-            return self.newcoords(self.rotation, self.translation)
+            rotation = np.matmul(matrix, self.rotation)
+            return self.newcoords(rotation, self.translation,
+                                  check_validity=False)
 
     def rotate(self, theta, axis, wrt='local'):
         """Rotate this coordinate.
@@ -1044,9 +1252,9 @@ class CascadedCoords(Coordinates):
         ----------
         theta : float
             radian
-        axis : string or numpy.ndarray
+        axis : str or numpy.ndarray
             'x', 'y', 'z' or vector
-        wrt : string or Coordinates
+        wrt : str or Coordinates
 
         Returns
         -------
@@ -1059,11 +1267,13 @@ class CascadedCoords(Coordinates):
             return self.rotate_with_matrix(theta, wrt)
 
         if wrt == 'local' or wrt == self:
-            self.rotation = rotate_matrix(self.rotation, theta, axis)
-            return self.newcoords(self.rotation, self.translation)
+            rotation = rotate_matrix(self.rotation, theta, axis)
+            return self.newcoords(rotation, self.translation,
+                                  check_validity=False)
         elif wrt == 'parent' or wrt == self.parent:
-            self.rotation = rotate_matrix(self.rotation, theta, axis)
-            return self.newcoords(self.rotation, self.translation)
+            rotation = rotate_matrix(self.rotation, theta, axis)
+            return self.newcoords(rotation, self.translation,
+                                  check_validity=False)
         else:
             return self.rotate_with_matrix(
                 rotation_matrix(theta, axis), wrt)
@@ -1107,13 +1317,15 @@ class CascadedCoords(Coordinates):
             # multiply c from the left.
             transform_coords(c, self, self)
         elif wrt == 'world':
-            transform_coords(self.parentcoords, self, self)
+            parentcoords = self.parentcoords()
+            transform_coords(parentcoords, self, self)
             transform_coords(c, self, self)
-            transform_coords(self.parentcoords.inverse_transformation(),
+            transform_coords(parentcoords.inverse_transformation(),
                              self, self)
         else:
             raise ValueError('transform wrt {} is not supported'.format(wrt))
-        return self.newcoords(self.rotation, self.translation)
+        return self.newcoords(self.rotation, self.translation,
+                              check_validity=False)
 
     def update(self, force=False):
         if not force and not self._changed:
@@ -1125,8 +1337,8 @@ class CascadedCoords(Coordinates):
                     self,
                     self._worldcoords)
             else:
-                self._worldcoords.rotation = self.rotation
-                self._worldcoords.translation = self.translation
+                self._worldcoords._rotation = self.rotation
+                self._worldcoords._translation = self.translation
         self._changed = False
 
     def worldcoords(self):
