@@ -1,5 +1,6 @@
 from __future__ import absolute_import
 
+import math
 from math import acos
 from math import asin
 from math import atan2
@@ -363,12 +364,17 @@ def midrot(p, r1, r2):
 def transform(m, v):
     """Return transform m v
 
-    Args:
-        m (numpy.ndarray): 3 x 3 matrix
-        v (numpy.ndarray or list): vector
+    Parameters
+    ----------
+    m : numpy.ndarray
+        3 x 3 rotation matrix.
+    v  : numpy.ndarray or list
+        input vector.
 
-    Returns:
-        numpy.array vector
+    Returns
+    -------
+    np.matmul(m, v) : numpy.ndarray
+        transformed vector.
     """
     m = np.array(m)
     v = np.array(v)
@@ -385,7 +391,7 @@ def rotation_matrix(theta, axis):
     ----------
     theta : float
         radian
-    axis : string or list or numpy.ndarray
+    axis : str or list or numpy.ndarray
         rotation axis such that 'x', 'y', 'z'
         [0, 0, 1], [0, 1, 0], [1, 0, 0]
 
@@ -429,7 +435,7 @@ def rotate_vector(vec, theta, axis):
         target vector
     theta : float
         rotation angle
-    axis : list or numpy.ndarray or string.
+    axis : list or numpy.ndarray or str
         axis of rotation.
 
     Returns
@@ -509,7 +515,7 @@ def rpy_angle(matrix):
 
     Returns
     -------
-    rpy : numpy.ndarray
+    rpy : tuple(numpy.ndarray, numpy.ndarray)
         pair of rpy in yaw-pitch-roll order.
 
     Examples
@@ -569,9 +575,10 @@ def normalize_vector(v, ord=2):
     array([0., 0., 0.])
     """
     v = np.array(v, dtype=np.float64)
-    if np.allclose(v, 0) is True:
+    norm = np.linalg.norm(v, ord=ord)
+    if norm == 0:
         return v
-    return v / np.linalg.norm(v, ord=ord)
+    return v / norm
 
 
 def matrix2quaternion(m):
@@ -595,34 +602,32 @@ def matrix2quaternion(m):
     array([1., 0., 0., 0.])
     """
     m = np.array(m, dtype=np.float64)
-    q0_2 = (1 + m[0, 0] + m[1, 1] + m[2, 2]) / 4.0
-    q1_2 = (1 + m[0, 0] - m[1, 1] - m[2, 2]) / 4.0
-    q2_2 = (1 - m[0, 0] + m[1, 1] - m[2, 2]) / 4.0
-    q3_2 = (1 - m[0, 0] - m[1, 1] + m[2, 2]) / 4.0
-    mq_2 = max(q0_2, q1_2, q2_2, q3_2)
-    if np.isclose(mq_2, q0_2):
-        q0 = np.sqrt(q0_2)
-        q1 = ((m[2, 1] - m[1, 2]) / (4.0 * q0))
-        q2 = ((m[0, 2] - m[2, 0]) / (4.0 * q0))
-        q3 = ((m[1, 0] - m[0, 1]) / (4.0 * q0))
-    elif np.isclose(mq_2, q1_2):
-        q1 = np.sqrt(q1_2)
-        q0 = ((m[2, 1] - m[1, 2]) / (4.0 * q1))
-        q2 = ((m[1, 0] + m[0, 1]) / (4.0 * q1))
-        q3 = ((m[0, 2] + m[2, 0]) / (4.0 * q1))
-    elif np.isclose(mq_2, q2_2):
-        q2 = np.sqrt(q2_2)
-        q0 = ((m[0, 2] - m[2, 0]) / (4.0 * q2))
-        q1 = ((m[1, 0] + m[0, 1]) / (4.0 * q2))
-        q3 = ((m[1, 2] + m[2, 1]) / (4.0 * q2))
-    elif np.isclose(mq_2, q3_2):
-        q3 = np.sqrt(q3_2)
-        q0 = ((m[1, 0] - m[0, 1]) / (4.0 * q3))
-        q1 = ((m[0, 2] + m[2, 0]) / (4.0 * q3))
-        q2 = ((m[1, 2] + m[2, 1]) / (4.0 * q3))
+    tr = m[0, 0] + m[1, 1] + m[2, 2]
+    if tr > 0:
+        S = math.sqrt(tr + 1.0) * 2
+        qw = 0.25 * S
+        qx = (m[2, 1] - m[1, 2]) / S
+        qy = (m[0, 2] - m[2, 0]) / S
+        qz = (m[1, 0] - m[0, 1]) / S
+    elif (m[0, 0] > m[1, 1]) and (m[0, 0] > m[2, 2]):
+        S = math.sqrt(1. + m[0, 0] - m[1, 1] - m[2, 2]) * 2
+        qw = (m[2, 1] - m[1, 2]) / S
+        qx = 0.25 * S
+        qy = (m[0, 1] + m[1, 0]) / S
+        qz = (m[0, 2] + m[2, 0]) / S
+    elif m[1, 1] > m[2, 2]:
+        S = math.sqrt(1. + m[1, 1] - m[0, 0] - m[2, 2]) * 2
+        qw = (m[0, 2] - m[2, 0]) / S
+        qx = (m[0, 1] + m[1, 0]) / S
+        qy = 0.25 * S
+        qz = (m[1, 2] + m[2, 1]) / S
     else:
-        raise ValueError('matrix {} is invalid'.format(m))
-    return np.array([q0, q1, q2, q3])
+        S = math.sqrt(1. + m[2, 2] - m[0, 0] - m[1, 1]) * 2
+        qw = (m[1, 0] - m[0, 1]) / S
+        qx = (m[0, 2] + m[2, 0]) / S
+        qy = (m[1, 2] + m[2, 1]) / S
+        qz = 0.25 * S
+    return np.array([qw, qx, qy, qz])
 
 
 def quaternion2matrix(q, normalize=False):
@@ -728,6 +733,9 @@ def matrix_log(m):
 def matrix_exponent(omega, p=1.0):
     """Returns exponent of given omega.
 
+    This function is similar to cv2.Rodrigues.
+    Convert rvec (which is log quaternion) to rotation matrix.
+
     Parameters
     ----------
     omega : list or numpy.ndarray
@@ -794,6 +802,24 @@ def outer_product_matrix(v):
                      [-v[1], v[0], 0]])
 
 
+def cross_product(a, b):
+    """Return cross product.
+
+    Parameters
+    ----------
+    a : numpy.ndarray
+        3-dimensional vector.
+    b : numpy.ndarray
+        3-dimensional vector.
+
+    Returns
+    -------
+    cross_prod : numpy.ndarray
+        calculated cross product
+    """
+    return np.dot(outer_product_matrix(a), b)
+
+
 def quaternion2rpy(q):
     """Returns Roll-pitch-yaw angles of a given quaternion.
 
@@ -925,9 +951,9 @@ def rotation_matrix_from_axis(
     e1 = normalize_vector(first_axis)
     e2 = normalize_vector(second_axis - np.dot(second_axis, e1) * e1)
     if axes in ['xy', 'zx', 'yz']:
-        third_axis = np.cross(e1, e2)
+        third_axis = cross_product(e1, e2)
     else:
-        third_axis = np.cross(e2, e1)
+        third_axis = cross_product(e2, e1)
     e3 = normalize_vector(
         third_axis - np.dot(third_axis, e1) * e1 - np.dot(third_axis, e2) * e2)
     first_index = ord(axes[0]) - ord('x')
@@ -1000,10 +1026,8 @@ def rotation_angle(mat):
 
     Returns
     -------
-    theta : float
-        rotation angle in radian
-    axis : numpy.ndarray
-        rotation axis
+    theta, axis : tuple(float, numpy.ndarray)
+        rotation angle in radian and rotation axis
 
     Examples
     --------
@@ -1388,6 +1412,27 @@ def quaternion_from_axis_angle(theta, axis):
     return np.array([w, x, y, z], dtype=np.float64)
 
 
+def rotation_vector_to_quaternion(rvec):
+    """Convert rotation vector to quaternion.
+
+    Parameters
+    ----------
+    rvec : list or numpy.ndarray
+        vector of shape (3,)
+
+    Returns
+    -------
+    q : numpy.ndarray
+        quaternion
+    """
+    rvec = np.array(rvec).reshape(-1)
+    theta = np.linalg.norm(rvec)
+    if theta == 0:
+        return np.array([1, 0, 0, 0], dtype=np.float64)
+    axis = rvec / theta
+    return quaternion_from_axis_angle(theta, axis)
+
+
 def axis_angle_from_quaternion(quat):
     """Converts a quaternion into the axis-angle representation.
 
@@ -1573,3 +1618,4 @@ quat_from_rotation_matrix = matrix2quaternion
 quat_from_rpy = rpy2quaternion
 rotation_matrix_from_quat = quaternion2matrix
 rpy_from_quat = quaternion2rpy
+rvec_to_quaternion = rotation_vector_to_quaternion
